@@ -1,6 +1,10 @@
 """Package 'tootbot' level definitions."""
-CODE_VERSION_MAJOR = 4  # Current major version of this code
+CODE_VERSION_MAJOR = 5  # Current major version of this code
 CODE_VERSION_MINOR = 0  # Current minor version of this code
 CODE_VERSION_PATCH = 0  # Current patch version of this code
 
 __version__ = f"{CODE_VERSION_MAJOR}.{CODE_VERSION_MINOR}.{CODE_VERSION_PATCH}"
+
+# Package level Static Variables
+POST_RECORDER_CSV_FILE = "cache.csv"
+POST_RECORDER_SQLITE_DB = "history.db"
