@@ -5,12 +5,13 @@ import time
 from alive_progress import alive_bar
 from rich import print as rprint
 
-from tootbot import __version__
-from tootbot.collect import LinkedMediaHelper
-from tootbot.collect import RedditHelper
-from tootbot.control import Configuration
-from tootbot.monitoring import HealthChecks
-from tootbot.publish import MastodonPublisher
+from . import __version__
+from .collect import get_secrets
+from .collect import LinkedMediaHelper
+from .collect import RedditHelper
+from .control import Configuration
+from .monitoring import HealthChecks
+from .publish import MastodonPublisher
 
 
 def main() -> None:
@@ -23,25 +24,37 @@ def main() -> None:
 
     rprint(f"Welcome to Tootbot ({__version__})")
 
+    secrets = get_secrets(logger=config.bot.logger)
+    MastodonPublisher.get_secrets(
+        mastodon_domain=config.mastodon_config.domain, logger=config.bot.logger
+    )
+
     title = "Setting up shop "
     with alive_bar(
         title=f"{title:.<60}",
         manual=True,
         enrich_print=False,
+        dual_line=True,
     ) as progress_bar:
 
         progress_bar.text = "Connecting to Mastodon instance ..."
         mastodon_publisher = MastodonPublisher(config=config)
         progress_bar(0.4120)  # pylint: disable=not-callable
 
+        progress_bar.text = "Connecting to Healthchecks ..."
         healthcheck = HealthChecks(config=config)
         progress_bar(0.4122)  # pylint: disable=not-callable
 
-        reddit = RedditHelper(config=config)
+        progress_bar.text = "Connecting to Reddit ..."
+        reddit = RedditHelper(config=config, api_secret=secrets["reddit"])
         progress_bar(0.4130)  # pylint: disable=not-callable
 
         progress_bar.text = "Connecting to Imgur and Gfycat ..."
-        media_helper = LinkedMediaHelper(config=config)
+        media_helper = LinkedMediaHelper(
+            config=config,
+            gfycat_secrets=secrets["gfycat"],
+            imgur_secrets=secrets["imgur"],
+        )
         progress_bar(1.0)  # pylint: disable=not-callable
 
     # Run the main script
@@ -49,12 +62,9 @@ def main() -> None:
         if config.health.enabled:
             healthcheck.check_start()
 
-        reddit_posts = {}
-        for subreddit in config.subreddits:
-            reddit_posts[subreddit.tags] = reddit.get_reddit_posts(
-                subreddit.name, limit=config.reddit.post_limit
-            )
-        mastodon_publisher.make_post(reddit_posts, reddit, media_helper)
+        reddit.get_all_reddit_posts()
+        reddit.winnow_reddit_posts()
+        mastodon_publisher.make_post(reddit.posts, reddit, media_helper)
 
         if config.health.enabled:
             healthcheck.check_ok()
@@ -80,7 +90,8 @@ def main() -> None:
 
         config.bot.logger.debug("Sleeping for %s seconds", sleep_time)
 
-        bar_title = "Sleeping before posting next toot"
+        rprint(" ")
+        bar_title = "Sleeping before next toot"
         with alive_bar(
             title=f"{bar_title:.<60}",
             total=sleep_time,
