@@ -11,6 +11,7 @@ from typing import Any
 from typing import Dict
 from typing import List
 from typing import Optional
+from typing import TypeVar
 from urllib.error import URLError
 from urllib.parse import urlsplit
 from urllib.request import urlopen
@@ -26,11 +27,17 @@ from imgurpython import ImgurClient
 from imgurpython.helpers.error import ImgurClientError
 from PIL import Image as PILImage
 from praw.models import Submission
+from rich import print as rprint
 
-from tootbot.control import Configuration
+from .control import Configuration
+from .control import Secret
 
 FATAL_TOOTBOT_ERROR = "Tootbot cannot continue, now shutting down"
 
+RH = TypeVar("RH", bound="RedditHelper")
+LMH = TypeVar("LMH", bound="LinkedMediaHelper")
+MA = TypeVar("MA", bound="MediaAttachment")
+
 
 # Function for downloading images from a URL to media folder
 def save_file(img_url: str, file_path: str, logger: logging.Logger) -> Optional[str]:
@@ -60,6 +67,172 @@ def save_file(img_url: str, file_path: str, logger: logging.Logger) -> Optional[
         return None
 
 
+def get_secrets(logger: logging.Logger) -> Dict[str, Secret]:
+    """Helper method to collect all api secrets either from already store
+    secrets files or from user input."""
+
+    secrets = {}
+    secrets["reddit"] = _get_reddit_secret(logger)
+    secrets["gfycat"] = _get_gfycat_secret(logger)
+    secrets["imgur"] = _get_imgur_secret(logger)
+
+    return secrets
+
+
+def _get_reddit_secret(logger: logging.Logger) -> Secret:
+    """Helper method to collect reddit secret from stored secrets file or from
+    user input."""
+    reddit_secrets_file = "reddit.secret"
+    reddit_config = configparser.ConfigParser()
+    if not os.path.exists(reddit_secrets_file):
+        rprint("Reddit API keys not found. Please provide Reddit API values.")
+        rprint("(See wiki if you need help).")
+        # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
+        reddit_agent = "".join(input("[ .. ] Enter Reddit agent: ").split())
+        reddit_client_secret = "".join(
+            input("[ .. ] Enter Reddit client secret: ").split()
+        )
+        # Make sure authentication is working
+        try:
+            # create Reddit api connection and load posts from announcements subreddit
+            # to confirm reddit connection works
+            reddit_client = praw.Reddit(
+                user_agent="Tootbot",
+                client_id=reddit_agent,
+                client_secret=reddit_client_secret,
+            )
+            subreddit = reddit_client.subreddit("announcements")
+            for _post in subreddit.hot():
+                continue
+
+            # It worked, so save the keys to a file
+            reddit_config["Reddit"] = {
+                "Agent": reddit_agent,
+                "ClientSecret": reddit_client_secret,
+            }
+            with open(reddit_secrets_file, "w", encoding="utf8") as new_reddit_secrets:
+                reddit_config.write(new_reddit_secrets)
+        except prawcore.exceptions.ResponseException as reddit_exception:
+            logger.error("Error while logging into Reddit: %s", reddit_exception)
+            logger.error(FATAL_TOOTBOT_ERROR)
+            sys.exit(1)
+    else:
+        # Read API keys from secret file
+        reddit_config.read(reddit_secrets_file)
+
+    return Secret(
+        client_id=reddit_config["Reddit"]["Agent"],
+        client_secret=reddit_config["Reddit"]["ClientSecret"],
+    )
+
+
+def _get_gfycat_secret(logger: logging.Logger) -> Secret:
+    """_get_gfycat_secrets checks if the Gfycat api secrets file exists.
+
+    - If the file exists, this method reads the files and returns the secrets
+      in a Secret dataclass.
+    - If the file doesn't exist it asks the user over stdin to supply these values
+      and then saves them into the gfycat_secrets file
+
+    Arguments:
+        logger (logging.Logger): logger to use for general logging messages
+
+    Returns:
+        api_secrets (Secret): instance of Secret class containing the api secrets
+        to work with gyfycat
+    """
+
+    secrets_file = "gfycat.secret"
+    if not os.path.exists(secrets_file):
+        rprint("Gfycat API keys not found. (See wiki if you need help).")
+
+        # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
+        gfycat_client_id = "".join(input("[ .. ] Enter Gfycat client ID: ").split())
+        gfycat_client_secret = "".join(
+            input("[ .. ] Enter Gfycat client secret: ").split()
+        )
+        # Make sure authentication is working
+        try:
+            gfycat_client = GfycatClient(gfycat_client_id, gfycat_client_secret)
+
+            # If this call doesn't work, it'll throw an ImgurClientError
+            gfycat_client.query_gfy("oddyearlyhorsefly")
+            # It worked, so save the keys to a file
+            gfycat_config = configparser.ConfigParser()
+            gfycat_config["Gfycat"] = {
+                "ClientID": gfycat_client_id,
+                "ClientSecret": gfycat_client_secret,
+            }
+            with open(secrets_file, "w", encoding="utf8") as file:
+                gfycat_config.write(file)
+        except GfycatClientError as gfycat_error:
+            logger.error("Error while logging into Gfycat: %s", gfycat_error)
+            logger.error(FATAL_TOOTBOT_ERROR)
+            sys.exit(1)
+    else:
+        # Read API keys from secret file
+        gfycat_config = configparser.ConfigParser()
+        gfycat_config.read(secrets_file)
+
+    return Secret(
+        client_id=gfycat_config["Gfycat"]["ClientID"],
+        client_secret=gfycat_config["Gfycat"]["ClientSecret"],
+    )
+
+
+def _get_imgur_secret(logger: logging.Logger) -> Secret:
+    """_get_imgur_secrets checks if the Imgur api secrets file exists.
+
+    - If the file exists, this method reads the imgur secrets file and returns the
+      secrets a Secret dataclass.
+    - If the file doesn't exist it asks the user over stdin to supply these values
+      and then saves them into the imgur_secrets file
+
+    Arguments:
+        logger (logging.Logger): logger to use for general logging messages
+
+    Returns:
+        api_secrets (Secret): instance of Secret class containing the api secrets
+        to work with imgut
+    """
+    secrets_file = "imgur.secret"
+    if not os.path.exists(secrets_file):
+        rprint("Imgur API keys not found. (See wiki if you need help).")
+
+        # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
+        imgur_client_id = "".join(input("[ .. ] Enter Imgur client ID: ").split())
+        imgur_client_secret = "".join(
+            input("[ .. ] Enter Imgur client secret: ").split()
+        )
+        # Make sure authentication is working
+        try:
+            imgur_client = ImgurClient(imgur_client_id, imgur_client_secret)
+
+            # If this call doesn't work, it'll throw an ImgurClientError
+            imgur_client.get_album("dqOyj")
+            # It worked, so save the keys to a file
+            imgur_config = configparser.ConfigParser()
+            imgur_config["Imgur"] = {
+                "ClientID": imgur_client_id,
+                "ClientSecret": imgur_client_secret,
+            }
+            with open(secrets_file, "w", encoding="UTF-8") as file:
+                imgur_config.write(file)
+        except ImgurClientError as imgur_error:
+            logger.error("Error while logging into Imgur: %s", imgur_error)
+            logger.error(FATAL_TOOTBOT_ERROR)
+            sys.exit(1)
+    else:
+        # Read API keys from secret file
+        imgur_config = configparser.ConfigParser()
+        imgur_config.read(secrets_file)
+
+    return Secret(
+        client_id=imgur_config["Imgur"]["ClientID"],
+        client_secret=imgur_config["Imgur"]["ClientSecret"],
+    )
+
+
 class RedditHelper:
     """RedditHelper provides methods to collect data / content from reddit to
     then post on Mastodon."""
@@ -67,123 +240,116 @@ class RedditHelper:
     # Check if reddit access details in 'reddit.secret' file has already been set up
     # and load it, otherwise guide user through setting it up.
     def __init__(
-        self,
+        self: RH,
         config: Configuration,
+        api_secret: Secret,
         user_agent: str = "Tootbot",
-        config_file: str = "reddit.secret",
-    ):
+    ) -> None:
+        self.config = config
         self.logger = config.bot.logger
-        self.user_agent = user_agent
-        self.reddit_config = config.reddit
-        self.global_hash_tags = config.bot.hash_tags
-        self.promo_every = config.promo.every
-        self.promo_message = config.promo.message
-
-        reddit_config = configparser.ConfigParser()
-        if not os.path.exists(config_file):
-            self.logger.warning(
-                "Reddit API keys not found. (See wiki if you need help)."
-            )
-            # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
-            reddit_agent = "".join(input("[ .. ] Enter Reddit agent: ").split())
-            reddit_client_secret = "".join(
-                input("[ .. ] Enter Reddit client secret: ").split()
-            )
-            # Make sure authentication is working
-            try:
-                reddit_client = praw.Reddit(
-                    user_agent=self.user_agent,
-                    client_id=reddit_agent,
-                    client_secret=reddit_client_secret,
-                )
-                reddit_client.subreddit("announcements")
-                # It worked, so save the keys to a file
-                reddit_config["Reddit"] = {
-                    "Agent": reddit_agent,
-                    "ClientSecret": reddit_client_secret,
-                }
-                with open(config_file, "w", encoding="utf8") as new_reddit_secrets_file:
-                    reddit_config.write(new_reddit_secrets_file)
-                new_reddit_secrets_file.close()
-            except prawcore.exceptions.ResponseException as reddit_exception:
-                self.logger.error(
-                    "Error while logging into Reddit: %s", reddit_exception
-                )
-                self.logger.error(FATAL_TOOTBOT_ERROR)
-                sys.exit(1)
-        else:
-            # Read API keys from secret file
-            reddit_config.read(config_file)
+        self.posts: Dict[str, praw.models.Submission] = {}
 
         self.reddit_connection = praw.Reddit(
-            user_agent=self.user_agent,
-            client_id=reddit_config["Reddit"]["Agent"],
-            client_secret=reddit_config["Reddit"]["ClientSecret"],
+            user_agent=user_agent,
+            client_id=api_secret.client_id,
+            client_secret=api_secret.client_secret,
         )
 
-    def get_reddit_posts(self, subreddit: str, limit: int = 10) -> Dict[str, Any]:
-        """get_reddit_posts reads up to "limit" number of posts from a given
-        subreddit and returns them as a dict of.
-
-        Arguments:
-            subreddit (string): name of subreddit (without leading "r/") to collect
-            posts from
-            limit (int): maximum number of posts to return (default 10)
-
-        Returns:
-            posts (dict): of posts to subreddit. each entry has a key of subreddit
-            post-id
-        """
-        posts: Dict[str, praw.models.Submission] = {}
-        subreddit_info = self.reddit_connection.subreddit(subreddit)
-        title = "Getting posts from Subreddit"
+    def get_all_reddit_posts(self: RH) -> None:
+        """Collects posts from all configured subreddits."""
+        title = "Collecting posts from reddit "
         with alive_bar(
-            title=f"{title:.<30}{subreddit:.>30}",
+            title=f"{title:.<60}",
+            enrich_print=False,
+            dual_line=True,
+        ) as progress_bar:
+            for subreddit in self.config.subreddits:
+                try:
+                    progress_bar.text = f"Getting posts from: r/{subreddit.name}"
+                    subreddit_info = self.reddit_connection.subreddit(subreddit.name)
+                    subreddit_posts: Dict[str, praw.models.Submission] = {}
+                    for submission in subreddit_info.hot(
+                        limit=self.config.reddit.post_limit
+                    ):
+                        subreddit_posts[submission.id] = submission
+                        progress_bar()  # pylint: disable=not-callable
+                    self.posts[subreddit.tags] = subreddit_posts
+                except (
+                    praw.exceptions.PRAWException,
+                    prawcore.exceptions.PrawcoreException,
+                ) as reddit_error:
+                    self.logger.warning(
+                        "Error when getting reddit posts from r/%s: %s",
+                        subreddit.name,
+                        reddit_error,
+                    )
+
+    def winnow_reddit_posts(self: RH) -> None:
+        """Filters out reddit posts according to configuration and whether it
+        has already been tooted."""
+        recorder = self.config.bot.post_recorder
+        nsfw_allowed = self.config.reddit.nsfw_allowed
+        self_posts_allowed = self.config.reddit.self_posts
+        spoilers_allowed = self.config.reddit.spoilers
+        stickied_allowed = self.config.reddit.stickied_allowed
+
+        title = "Winnowing chaff "
+        with alive_bar(
+            title=f"{title:.<60}",
             enrich_print=False,
         ) as progress_bar:
+            for posts in self.posts.values():
+                post_ids = list(posts.keys())
+                for post_id in post_ids:
 
-            try:
-                for submission in subreddit_info.hot(limit=limit):
-                    progress_bar.text = f"checking redit post with id {submission.id}"
-
-                    if submission.over_18 and not self.reddit_config.nsfw_allowed:
+                    if posts[post_id].over_18 and not nsfw_allowed:
                         # Skip over NSFW posts if they are disabled in the config file
-                        self.logger.debug(
-                            "Skipping %s, it is marked as NSFW", submission.id
-                        )
+                        self.logger.debug("Skipping %s, it is marked as NSFW", post_id)
+                        del posts[post_id]
+                        progress_bar()  # pylint: disable=not-callable
                         continue
 
-                    if submission.is_self and not self.reddit_config.self_posts:
+                    if posts[post_id].is_self and not self_posts_allowed:
                         # Skip over NSFW posts if they are disabled in the config file
-                        self.logger.debug(
-                            "Skipping %s, it is a self post", submission.id
-                        )
+                        self.logger.debug("Skipping %s, it is a self post", post_id)
+                        del posts[post_id]
+                        progress_bar()  # pylint: disable=not-callable
                         continue
 
-                    if submission.spoiler and not self.reddit_config.spoilers:
+                    if posts[post_id].spoiler and not spoilers_allowed:
                         # Skip over posts marked as spoilers if they are disabled in
                         # the config file
                         self.logger.debug(
-                            "Skipping %s, it is marked as a spoiler", submission.id
+                            "Skipping %s, it is marked as a spoiler", post_id
                         )
+                        del posts[post_id]
+                        progress_bar()  # pylint: disable=not-callable
                         continue
 
-                    if submission.stickied and not self.reddit_config.stickied_allowed:
-                        self.logger.debug("Skipping %s, it is stickied", submission.id)
+                    if posts[post_id].stickied and not stickied_allowed:
+                        self.logger.debug("Skipping %s, it is stickied", post_id)
+                        del posts[post_id]
+                        progress_bar()  # pylint: disable=not-callable
                         continue
 
-                    # Create dict
-                    posts[submission.id] = submission
-                    progress_bar()  # pylint: disable=not-callable
-            except prawcore.exceptions.ResponseException as reddit_exception:
-                self.logger.debug(
-                    "Encountered an error getting reddit posts: %s", reddit_exception
-                )
+                    if recorder.duplicate_check(post_id):
+                        self.logger.debug(
+                            "Skipping %s, it has already been tooted", post_id
+                        )
+                        del posts[post_id]
+                        progress_bar()  # pylint: disable=not-callable
+                        continue
 
-        return posts
+                    if recorder.duplicate_check(posts[post_id].url):
+                        self.logger.debug(
+                            "Skipping %s, it has already been tooted", post_id
+                        )
+                        del posts[post_id]
+                        progress_bar()  # pylint: disable=not-callable
+                        continue
 
     def get_caption(
-        self,
+        self: RH,
         submission: Submission,
         max_len: int,
         add_hash_tags: Optional[str] = None,
@@ -206,19 +372,19 @@ class RedditHelper:
         # Create string of hashtags
         hashtag_string = ""
         promo_string = ""
-        hashtags_for_post = self.global_hash_tags
+        hashtags_for_post = self.config.bot.hash_tags
 
         # Workout hashtags for post
         if add_hash_tags is not None:
             hashtags_for_subreddit = [x.strip() for x in add_hash_tags.split(",")]
-            hashtags_for_post = hashtags_for_subreddit + self.global_hash_tags
+            hashtags_for_post = hashtags_for_subreddit + self.config.bot.hash_tags
         if hashtags_for_post:
             for tag in hashtags_for_post:
                 # Add hashtag to string, followed by a space for the next one
                 hashtag_string += "#" + tag + " "
 
         if promo_message:
-            promo_string = f" \n \n{self.promo_message}"
+            promo_string = f" \n \n{self.config.promo.message}"
         caption_max_length = max_len
         caption_max_length -= (
             len(submission.shortlink) - len(hashtag_string) - len(promo_string)
@@ -238,27 +404,23 @@ class LinkedMediaHelper:
     Gfycat."""
 
     def __init__(
-        self,
+        self: LMH,
         config: Configuration,
-        imgur_secrets: str = "imgur.secret",
-        gfycat_secrets: str = "gfycat.secret",
-    ):
+        gfycat_secrets: Secret,
+        imgur_secrets: Secret,
+    ) -> None:
         self.logger = config.bot.logger
         self.save_dir = config.media.folder
 
         try:
-            imgur_config = self._get_imgur_secrets(imgur_secrets)
             self.imgur_client = ImgurClient(
-                imgur_config["Imgur"]["ClientID"],
-                imgur_config["Imgur"]["ClientSecret"],
+                client_id=imgur_secrets.client_id,
+                client_secret=imgur_secrets.client_secret,
             )
-
-            gfycat_config = self._get_gfycat_secrets(gfycat_secrets)
             self.gfycat_client = GfycatClient(
-                gfycat_config["Gfycat"]["ClientID"],
-                gfycat_config["Gfycat"]["ClientSecret"],
+                client_id=gfycat_secrets.client_id,
+                client_secret=gfycat_secrets.client_secret,
             )
-
         except ImgurClientError as imgur_error:
             self.logger.error("Error on creating ImgurClient: %s", imgur_error)
             self.logger.error(FATAL_TOOTBOT_ERROR)
@@ -268,111 +430,7 @@ class LinkedMediaHelper:
             self.logger.error(FATAL_TOOTBOT_ERROR)
             sys.exit(1)
 
-    def _get_gfycat_secrets(self, gfycat_secrets: str) -> configparser.ConfigParser:
-        """_get_gfycat_secrets checks if the Gfycat api secrets file exists.
-
-        - If the file exists, this method reads the files and returns the secrets
-          in as a dict.
-        - If the file doesn't exist it asks the user over stdin to supply these values
-          and then saves them into the gfycat_secrets file
-
-        Arguments:
-            gfycat_secrets (string): file name of secrets file for API credentials
-
-        Returns:
-            imgur_config (dict): Dictionary containing the client id and client secret
-            needed to log into Gfycat
-        """
-
-        if not os.path.exists(gfycat_secrets):
-            self.logger.warning(
-                "Gfycat API keys not found. (See wiki if you need help)."
-            )
-
-            # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
-            gfycat_client_id = "".join(input("[ .. ] Enter Gfycat client ID: ").split())
-            gfycat_client_secret = "".join(
-                input("[ .. ] Enter Gfycat client secret: ").split()
-            )
-            # Make sure authentication is working
-            try:
-                gfycat_client = GfycatClient(gfycat_client_id, gfycat_client_secret)
-
-                # If this call doesn't work, it'll throw an ImgurClientError
-                gfycat_client.query_gfy("oddyearlyhorsefly")
-                # It worked, so save the keys to a file
-                gfycat_config = configparser.ConfigParser()
-                gfycat_config["Gfycat"] = {
-                    "ClientID": gfycat_client_id,
-                    "ClientSecret": gfycat_client_secret,
-                }
-                with open(gfycat_secrets, "w", encoding="utf8") as file:
-                    gfycat_config.write(file)
-                file.close()
-            except GfycatClientError as gfycat_error:
-                self.logger.error("Error while logging into Gfycat: %s", gfycat_error)
-                self.logger.error(FATAL_TOOTBOT_ERROR)
-                sys.exit(1)
-        else:
-            # Read API keys from secret file
-            gfycat_config = configparser.ConfigParser()
-            gfycat_config.read(gfycat_secrets)
-
-        return gfycat_config
-
-    def _get_imgur_secrets(self, imgur_secrets: str) -> configparser.ConfigParser:
-        """_get_imgur_secrets checks if the Imgur api secrets file exists.
-
-        - If the file exists, this method reads the imgur secrets file and returns the
-          secrets in as a dict.
-        - If the file doesn't exist it asks the user over stdin to supply these values
-          and then saves them into the imgur_secrets file
-
-        Arguments:
-            imgur_secrets (string): file name of secrets file for API credentials
-
-        Returns:
-            imgur_config (dict): Dictionary containing the client id and client secret
-            needed to log into Imgur
-        """
-
-        if not os.path.exists(imgur_secrets):
-            self.logger.warning(
-                "Imgur API keys not found. (See wiki if you need help)."
-            )
-
-            # Whitespaces are stripped from input: https://stackoverflow.com/a/3739939
-            imgur_client_id = "".join(input("[ .. ] Enter Imgur client ID: ").split())
-            imgur_client_secret = "".join(
-                input("[ .. ] Enter Imgur client secret: ").split()
-            )
-            # Make sure authentication is working
-            try:
-                imgur_client = ImgurClient(imgur_client_id, imgur_client_secret)
-
-                # If this call doesn't work, it'll throw an ImgurClientError
-                imgur_client.get_album("dqOyj")
-                # It worked, so save the keys to a file
-                imgur_config = configparser.ConfigParser()
-                imgur_config["Imgur"] = {
-                    "ClientID": imgur_client_id,
-                    "ClientSecret": imgur_client_secret,
-                }
-                with open(imgur_secrets, "w", encoding="UTF-8") as file:
-                    imgur_config.write(file)
-                file.close()
-            except ImgurClientError as imgur_error:
-                self.logger.error("Error while logging into Imgur: %s", imgur_error)
-                self.logger.error(FATAL_TOOTBOT_ERROR)
-                sys.exit(1)
-        else:
-            # Read API keys from secret file
-            imgur_config = configparser.ConfigParser()
-            imgur_config.read(imgur_secrets)
-
-        return imgur_config
-
-    def get_imgur_image(self, img_url: str, max_images: int = 4) -> List[str]:
+    def get_imgur_image(self: LMH, img_url: str, max_images: int = 4) -> List[str]:
         """get_imgur_image downloads images from imgur.
 
         Arguments:
@@ -447,7 +505,7 @@ class LinkedMediaHelper:
 
         return imgur_paths
 
-    def _get_image_urls(self, img_url: str, imgur_id: str) -> List[str]:
+    def _get_image_urls(self: LMH, img_url: str, imgur_id: str) -> List[str]:
         """_get_image_urls builds a list of urls of all Imgur images identified
         by imgur_id.
 
@@ -473,7 +531,7 @@ class LinkedMediaHelper:
             self.logger.error("Could not get information from imgur: %s", imgur_error)
         return image_urls
 
-    def _check_imgur_gif(self, file_path: str) -> bool:
+    def _check_imgur_gif(self: LMH, file_path: str) -> bool:
         """_check_imgur_gif checks if a file downloaded from imgur is indeed a
         gif. If file is not a gif, remove the file.
 
@@ -497,7 +555,7 @@ class LinkedMediaHelper:
 
         return True
 
-    def get_gfycat_image(self, img_url: str) -> Optional[str]:
+    def get_gfycat_image(self: LMH, img_url: str) -> Optional[str]:
         """get_gfycat_image downloads full resolution images from gfycat.
 
         Arguments:
@@ -545,7 +603,7 @@ class LinkedMediaHelper:
 
         return saved_file
 
-    def get_reddit_image(self, img_url: str) -> Optional[str]:
+    def get_reddit_image(self: LMH, img_url: str) -> Optional[str]:
         """get_reddit_image downloads full resolution images from i.reddit or
         reddituploads.com.
 
@@ -587,7 +645,9 @@ class LinkedMediaHelper:
         return saved_file
 
     def get_reddit_gallery(
-        self, reddit_post: Submission, max_images: int = 4
+        self: LMH,
+        reddit_post: Submission,
+        max_images: int = 4,
     ) -> List[Optional[str]]:
         """get_reddit_gallery downloads up to max_images images from a reddit
         gallery post and returns a List of file_paths downloaded images.
@@ -623,9 +683,6 @@ class LinkedMediaHelper:
                     self.logger.debug(
                         "Gallery file_path, source: %s - %s", save_path, source["u"]
                     )
-                    self.logger.debug(
-                        "A[%4dx%04d] %s", (source["x"], source["y"], source["u"])
-                    )
                     progress_bar.text = f"{save_path} <-- {source['u']}"
                     file_paths.append(save_file(source["u"], save_path, self.logger))
 
@@ -636,7 +693,7 @@ class LinkedMediaHelper:
 
         return file_paths
 
-    def get_reddit_video(self, reddit_post: Submission) -> Optional[str]:
+    def get_reddit_video(self: LMH, reddit_post: Submission) -> Optional[str]:
         """get_reddit_video downloads full resolution video from i.reddit or
         reddituploads.
 
@@ -666,7 +723,7 @@ class LinkedMediaHelper:
 
         return saved_file
 
-    def get_giphy_image(self, img_url: str) -> Optional[str]:
+    def get_giphy_image(self: LMH, img_url: str) -> Optional[str]:
         """get_giphy_image downloads full or low resolution image from giphy.
 
         Arguments:
@@ -707,7 +764,7 @@ class LinkedMediaHelper:
 
         return giphy_file
 
-    def get_generic_image(self, img_url: str) -> Optional[str]:
+    def get_generic_image(self: LMH, img_url: str) -> Optional[str]:
         """get_generic_image downloads image or video from a generic url to a
         media file.
 
@@ -734,7 +791,9 @@ class LinkedMediaHelper:
             "video/mp4",
         )
         try:
-            with urlopen(img_url) as img_site:
+            # img_url has already been checked that it starts with "http://" or
+            # "https://" above.
+            with urlopen(img_url) as img_site:  # nosec B310
                 meta = img_site.info()
 
         except (URLError, UnicodeEncodeError) as url_error:
@@ -769,11 +828,11 @@ class MediaAttachment:
     videos to include in a reddit post to be shared on Mastodon."""
 
     def __init__(
-        self,
+        self: MA,
         reddit_post: Submission,
         image_helper: LinkedMediaHelper,
         logger: logging.Logger,
-    ):
+    ) -> None:
 
         self.media_paths = {}
         self.reddit_post = reddit_post
@@ -781,6 +840,8 @@ class MediaAttachment:
         self.image_helper = image_helper
         self.logger = logger
 
+        logger.debug("URL for post(%s): %s ", reddit_post, self.media_url)
+
         for media_path in self.get_media():
             self.logger.debug("Media path for checksum calculation: %s", media_path)
             if media_path:
@@ -795,7 +856,7 @@ class MediaAttachment:
 
                 self.media_paths[sha256.hexdigest()] = media_path
 
-    def destroy(self):
+    def destroy(self: MA) -> None:
         """Removes any files downloaded and clears out the object
         attributes."""
         try:
@@ -809,7 +870,7 @@ class MediaAttachment:
         self.media_paths = {}
         self.media_url = None
 
-    def destroy_one_attachment(self, checksum: str):
+    def destroy_one_attachment(self: MA, checksum: str) -> None:
         """Removes file with checksum downloaded.
 
         Arguments:
@@ -824,8 +885,7 @@ class MediaAttachment:
         except OSError as delete_error:
             self.logger.error("Error while deleting media file: %s", delete_error)
 
-    # Function for obtaining static images and GIFs from popular image hosts
-    def get_media(self) -> List[Optional[str]]:
+    def get_media(self: MA) -> List[Optional[str]]:
         """Determines which method to call depending on which site the
         media_url is pointing to."""
         if not os.path.exists(self.image_helper.save_dir):
@@ -838,10 +898,7 @@ class MediaAttachment:
         file_paths = []
 
         # Download and save the linked image
-        if hasattr(self.reddit_post, "is_gallery"):
-            self.logger.debug("%s is a gallery post", self.reddit_post.id)
-            file_paths.extend(self.image_helper.get_reddit_gallery(self.reddit_post))
-        elif any(s in self.media_url for s in ("i.redd.it", "i.reddituploads.com")):
+        if any(s in self.media_url for s in ("i.redd.it", "i.reddituploads.com")):
             file_paths.append(self.image_helper.get_reddit_image(self.media_url))
         elif "v.redd.it" in self.media_url and not self.reddit_post.media:
             self.logger.error(
@@ -854,12 +911,19 @@ class MediaAttachment:
             self.logger.debug("Reddit post %s links to Imgur", self.reddit_post.id)
             file_paths.extend(self.image_helper.get_imgur_image(self.media_url))
 
-        elif "gfycat.com" in self.media_url:  # Gfycat
+        elif "gfycat.com" in self.media_url:
             file_paths.append(self.image_helper.get_gfycat_image(self.media_url))
 
-        elif "giphy.com" in self.media_url:  # Giphy
+        elif "giphy.com" in self.media_url:
             file_paths.append(self.image_helper.get_giphy_image(self.media_url))
 
+        elif "reddit.com/gallery/" in self.media_url:  # Need to check for gallery post
+            if hasattr(self.reddit_post, "is_gallery"):
+                self.logger.debug("%s is a gallery post", self.reddit_post.id)
+                file_paths.extend(
+                    self.image_helper.get_reddit_gallery(self.reddit_post)
+                )
+
         else:
             file_paths.append(self.image_helper.get_generic_image(self.media_url))
 
