@@ -1,7 +1,7 @@
 """Provides link between command and code."""
-from tootbot.app import main
+from .app import main
 
 
-def start():
+def start() -> None:
     """Starts actual code."""
     main()
